"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudWatchEnvelope = void 0;
const errors_js_1 = require("../errors.js");
const index_js_1 = require("../schemas/index.js");
const envelope_js_1 = require("./envelope.js");
/**
 * CloudWatch Envelope to extract a List of log records.
 *
 *  The record's body parameter is a string (after being base64 decoded and gzipped),
 *  though it can also be a JSON encoded string.
 *  Regardless of its type it'll be parsed into a BaseModel object.
 *
 *  Note: The record will be parsed the same way so if model is str
 */
exports.CloudWatchEnvelope = {
    /**
     * This is a discriminator to differentiate whether an envelope returns an array or an object
     * @hidden
     */
    [envelope_js_1.envelopeDiscriminator]: 'array',
    parse(data, schema) {
        const parsedEnvelope = index_js_1.CloudWatchLogsSchema.parse(data);
        return parsedEnvelope.awslogs.data.logEvents.map((record) => {
            return envelope_js_1.Envelope.parse(record.message, schema);
        });
    },
    safeParse(data, schema) {
        const parsedEnvelope = index_js_1.CloudWatchLogsSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new errors_js_1.ParseError('Failed to parse CloudWatch envelope', {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedLogEvents = [];
        for (const record of parsedEnvelope.data.awslogs.data.logEvents) {
            const parsedMessage = envelope_js_1.Envelope.safeParse(record.message, schema);
            if (!parsedMessage.success) {
                return {
                    success: false,
                    error: new errors_js_1.ParseError('Failed to parse CloudWatch log event', {
                        cause: parsedMessage.error,
                    }),
                    originalEvent: data,
                };
            }
            parsedLogEvents.push(parsedMessage.data);
        }
        return {
            success: true,
            data: parsedLogEvents,
        };
    },
};
