"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqsEnvelope = void 0;
const errors_js_1 = require("../errors.js");
const sqs_js_1 = require("../schemas/sqs.js");
const envelope_js_1 = require("./envelope.js");
/**
 *  SQS Envelope to extract array of Records
 *
 *  The record's body parameter is a string, though it can also be a JSON encoded string.
 *  Regardless of its type it'll be parsed into a BaseModel object.
 *
 *  Note: Records will be parsed the same way so if model is str,
 *  all items in the list will be parsed as str and npt as JSON (and vice versa)
 */
exports.SqsEnvelope = {
    /**
     * This is a discriminator to differentiate whether an envelope returns an array or an object
     * @hidden
     */
    [envelope_js_1.envelopeDiscriminator]: 'array',
    parse(data, schema) {
        const parsedEnvelope = sqs_js_1.SqsSchema.parse(data);
        return parsedEnvelope.Records.map((record) => {
            return envelope_js_1.Envelope.parse(record.body, schema);
        });
    },
    safeParse(data, schema) {
        const parsedEnvelope = sqs_js_1.SqsSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new errors_js_1.ParseError('Failed to parse Sqs Envelope', {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedRecords = [];
        for (const record of parsedEnvelope.data.Records) {
            const parsedRecord = envelope_js_1.Envelope.safeParse(record.body, schema);
            if (!parsedRecord.success) {
                return {
                    success: false,
                    error: new errors_js_1.ParseError('Failed to parse Sqs Record', {
                        cause: parsedRecord.error,
                    }),
                    originalEvent: data,
                };
            }
            parsedRecords.push(parsedRecord.data);
        }
        return { success: true, data: parsedRecords };
    },
};
