"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserIdentity = exports.DynamoDBStreamChangeRecord = exports.DynamoDBStreamRecord = exports.DynamoDBStreamSchema = exports.DynamoDBStreamToKinesisRecord = void 0;
const zod_1 = require("zod");
const DynamoDBStreamChangeRecord = zod_1.z.object({
    ApproximateCreationDateTime: zod_1.z.number().optional(),
    Keys: zod_1.z.record(zod_1.z.string(), zod_1.z.record(zod_1.z.string(), zod_1.z.any())),
    NewImage: zod_1.z.record(zod_1.z.string(), zod_1.z.any()).optional(),
    OldImage: zod_1.z.record(zod_1.z.string(), zod_1.z.any()).optional(),
    SequenceNumber: zod_1.z.string(),
    SizeBytes: zod_1.z.number(),
    StreamViewType: zod_1.z.enum([
        'NEW_IMAGE',
        'OLD_IMAGE',
        'NEW_AND_OLD_IMAGES',
        'KEYS_ONLY',
    ]),
});
exports.DynamoDBStreamChangeRecord = DynamoDBStreamChangeRecord;
const UserIdentity = zod_1.z.object({
    type: zod_1.z.enum(['Service']),
    principalId: zod_1.z.literal('dynamodb.amazonaws.com'),
});
exports.UserIdentity = UserIdentity;
const DynamoDBStreamRecord = zod_1.z.object({
    eventID: zod_1.z.string(),
    eventName: zod_1.z.enum(['INSERT', 'MODIFY', 'REMOVE']),
    eventVersion: zod_1.z.string(),
    eventSource: zod_1.z.literal('aws:dynamodb'),
    awsRegion: zod_1.z.string(),
    eventSourceARN: zod_1.z.string(),
    dynamodb: DynamoDBStreamChangeRecord,
    userIdentity: UserIdentity.optional(),
});
exports.DynamoDBStreamRecord = DynamoDBStreamRecord;
const DynamoDBStreamToKinesisRecord = DynamoDBStreamRecord.extend({
    recordFormat: zod_1.z.literal('application/json'),
    tableName: zod_1.z.string(),
    userIdentity: UserIdentity.nullish(),
    dynamodb: DynamoDBStreamChangeRecord.omit({
        SequenceNumber: true,
        StreamViewType: true,
    }),
}).omit({
    eventVersion: true,
    eventSourceARN: true,
});
exports.DynamoDBStreamToKinesisRecord = DynamoDBStreamToKinesisRecord;
/**
 * Zod schema for Amazon DynamoDB Stream event.
 *
 * @example
 * ```json
 * {
 *   "Records": [
 *     {
 *       "eventID": "1",
 *       "eventVersion": "1.0",
 *       "dynamodb": {
 *         "ApproximateCreationDateTime": 1693997155.0,
 *         "Keys": {
 *           "Id": {
 *             "N": "101"
 *           }
 *         },
 *         "NewImage": {
 *           "Message": {
 *             "S": "New item!"
 *           },
 *           "Id": {
 *             "N": "101"
 *           }
 *         },
 *         "StreamViewType": "NEW_AND_OLD_IMAGES",
 *         "SequenceNumber": "111",
 *         "SizeBytes": 26
 *       },
 *       "awsRegion": "us-west-2",
 *       "eventName": "INSERT",
 *       "eventSourceARN": "eventsource_arn",
 *       "eventSource": "aws:dynamodb"
 *     },
 *     {
 *       "eventID": "2",
 *       "eventVersion": "1.0",
 *       "dynamodb": {
 *         "OldImage": {
 *           "Message": {
 *             "S": "New item!"
 *           },
 *           "Id": {
 *             "N": "101"
 *           }
 *         },
 *         "SequenceNumber": "222",
 *         "Keys": {
 *           "Id": {
 *             "N": "101"
 *           }
 *         },
 *         "SizeBytes": 59,
 *         "NewImage": {
 *           "Message": {
 *             "S": "This item has changed"
 *           },
 *           "Id": {
 *             "N": "101"
 *           }
 *         },
 *         "StreamViewType": "NEW_AND_OLD_IMAGES"
 *       },
 *       "awsRegion": "us-west-2",
 *       "eventName": "MODIFY",
 *       "eventSourceARN": "source_arn",
 *       "eventSource": "aws:dynamodb"
 *     }
 *   ]
 * }
 * ```
 *
 * @see {@link types.DynamoDBStreamEvent | DynamoDBStreamEvent}
 * @see {@link https://docs.aws.amazon.com/lambda/latest/dg/with-ddb.html}
 */
const DynamoDBStreamSchema = zod_1.z.object({
    Records: zod_1.z.array(DynamoDBStreamRecord),
});
exports.DynamoDBStreamSchema = DynamoDBStreamSchema;
