import { ParseError } from '../errors.js';
import { SnsSchema, SnsSqsNotificationSchema } from '../schemas/sns.js';
import { SqsSchema } from '../schemas/sqs.js';
import { Envelope, envelopeDiscriminator } from './envelope.js';
/**
 * SNS Envelope to extract array of Records
 *
 * The record's body parameter is a string, though it can also be a JSON encoded string.
 * Regardless of its type it'll be parsed into a BaseModel object.
 *
 * Note: Records will be parsed the same way so if model is str,
 * all items in the list will be parsed as str and npt as JSON (and vice versa)
 */
export const SnsEnvelope = {
    /**
     * This is a discriminator to differentiate whether an envelope returns an array or an object
     * @hidden
     */
    [envelopeDiscriminator]: 'array',
    parse(data, schema) {
        const parsedEnvelope = SnsSchema.parse(data);
        return parsedEnvelope.Records.map((record) => {
            return Envelope.parse(record.Sns.Message, schema);
        });
    },
    safeParse(data, schema) {
        const parsedEnvelope = SnsSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new ParseError('Failed to parse SNS envelope', {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedMessages = [];
        for (const record of parsedEnvelope.data.Records) {
            const parsedMessage = Envelope.safeParse(record.Sns.Message, schema);
            if (!parsedMessage.success) {
                return {
                    success: false,
                    error: new ParseError('Failed to parse SNS message', {
                        cause: parsedMessage.error,
                    }),
                    originalEvent: data,
                };
            }
            parsedMessages.push(parsedMessage.data);
        }
        return {
            success: true,
            data: parsedMessages,
        };
    },
};
/**
 *  SNS plus SQS Envelope to extract array of Records
 *
 *  Published messages from SNS to SQS has a slightly different payload.
 *  Since SNS payload is marshalled into `Record` key in SQS, we have to:
 *
 *  1. Parse SQS schema with incoming data
 *  2. Unmarshall SNS payload and parse against SNS Notification schema not SNS/SNS Record
 *  3. Finally, parse provided model against payload extracted
 *
 */
export const SnsSqsEnvelope = {
    /**
     * This is a discriminator to differentiate whether an envelope returns an array or an object
     * @hidden
     */
    [envelopeDiscriminator]: 'array',
    parse(data, schema) {
        const parsedEnvelope = SqsSchema.parse(data);
        return parsedEnvelope.Records.map((record) => {
            const snsNotification = SnsSqsNotificationSchema.parse(JSON.parse(record.body));
            return Envelope.parse(snsNotification.Message, schema);
        });
    },
    safeParse(data, schema) {
        const parsedEnvelope = SqsSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new ParseError('Failed to parse SQS envelope', {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedMessages = [];
        // JSON.parse can throw an error, thus we catch it and return ParsedErrorResult
        try {
            for (const record of parsedEnvelope.data.Records) {
                const snsNotification = SnsSqsNotificationSchema.safeParse(JSON.parse(record.body));
                if (!snsNotification.success) {
                    return {
                        success: false,
                        error: new ParseError('Failed to parse SNS notification', {
                            cause: snsNotification.error,
                        }),
                        originalEvent: data,
                    };
                }
                const parsedMessage = Envelope.safeParse(snsNotification.data.Message, schema);
                if (!parsedMessage.success) {
                    return {
                        success: false,
                        error: new ParseError('Failed to parse SNS message', {
                            cause: parsedMessage.error,
                        }),
                        originalEvent: data,
                    };
                }
                parsedMessages.push(parsedMessage.data);
            }
        }
        catch (e) {
            return {
                success: false,
                error: e,
                originalEvent: data,
            };
        }
        return { success: true, data: parsedMessages };
    },
};
